<?php
// +----------------------------------------------------------------------
// 小黄牛blog - Swoole 即时通讯交互处理
// +----------------------------------------------------------------------
// Copyright (c) 2018 https://xiuxian.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

class Server{
	/**
	 * 客户端身份存储器
	*/
	private $_data = []; 
	/**
	 * WS的启动实例
	*/
	private $_ws;
	/**
	 * host-IP，0.0.0.0表示允许接收所有请求
	*/
	private $_host = '0.0.0.0';
	/**
	 * 端口号
	*/
	private $_port = '9502';

    /**
     * 这是启动服务端的入口
    */
    public function run() { 
		$this->start_service(); 
		$this->start_handshake();
		$this->start_message();
		$this->end();
    }

	/**
	 * ①启动websocker服务
	*/
	private function start_service() {
		# 创建websocket服务器对象，监听0.0.0.0:9502端口
		$this->_ws = new swoole_websocket_server($this->_host, $this->_port);
	}

	/**
	 * ②监听WebSocket握手申请
	*/
	private function start_handshake() {
		# 监听WebSocket连接打开事件
		$this->_ws->on('open', function ($ws, $request){
			# 加入身份存储器
			$this->_data[$request->fd]['fd']      = $request->fd;
			# 默认离线状态
			$this->_data[$request->fd]['status']  = 0;
		});
	}

	/**
	 * ③监听客户端消息发送请求
	*/
	private function start_message() {
		# 监听WebSocket消息事件
		$this->_ws->on('message', function ($ws, $frame) {
			$data = json_decode($frame->data, true);
			# 登录广播处理
			if ($data['code'] == 1) {
				# 修改fd对应的身份
				$this->_data[$frame->fd]['user_id'] = $data['user_id'];
				# 设置上线状态
				$this->_data[$frame->fd]['status']  = 1;
				# 发送广播上线消息
				$data['content'] = '【'.$data['user_id'].'】骑着小黄牛上线啦~！';
				$this->broadcast($ws, $this->json($data), $frame->fd);

			# 其他请求
			} else {
				# 广播消息
				$this->broadcast($ws, $frame->data, $frame->fd);
			}
		});
	} 

	/**
	 * ④监听客户端退出事件
	*/
	private function end() {
		# 监听WebSocket连接关闭事件
		$this->_ws->on('close', function ($ws, $fd) {
			# 获取用户ID
			$user_id = $this->_data[$fd]['user_id'];
			# 设置离线状态
			$this->_data[$fd]['status']  = 0;
			$data = [
				'code' => 2,
				'user_id' => $user_id,
				'content' => '【'.$user_id.'】骑着小扫帚灰溜溜的走了~~！'
			];
			# 广播消息
			$this->broadcast($ws, $this->json($data));
		});

		$this->_ws->start();
	}

	/**
	 * 广播消息
	 * @todo 无
	 * @author 小黄牛
	 * @version v1.0.0.1 + 2018.11.12
	 * @deprecated 暂不弃用
	 * @global 无
	 * @param object $wx 实例
	 * @param string $content 广播内容
	 * @param string $id 自己的身份凭证 如果为空则也向自己广播
	 * @return void
	*/
	private function broadcast($ws, $content, $id=null) {
		# 向所有人广播
		foreach ($this->_data as $k=>$v) {
			# 不向自己广播，并且要在线的
			# 注意，这里一定要有上线状态的限制，否则假设用户已经退出，但你的进程还开着，实际上已经关闭，这时候push就会报错
			if ($k != $id && $v['status'] == 1) {
				$ws->push($v['fd'], $content);
			}
		}
	}

	/**
	 * 数组转json
	 * @todo 无
	 * @author 小黄牛
	 * @version v1.0.0.1 + 2018.11.08
	 * @deprecated 暂不弃用
	 * @global 无
	 * @param array $array 数组
	 * @return json
	*/
	private function json($array) {
		return json_encode($array, JSON_UNESCAPED_UNICODE);
	}
}

$socketServer = new Server();
$socketServer->run();